/*
 * Copyright (c) 2020 Jetro AS.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <drivers/entropy.h>
#include <drivers/counter.h>
#include <stdio.h>

#define SPM_PRIORITY	7
#define SPM_STACK_SIZE	4096
static K_THREAD_STACK_DEFINE(spm_stack_area, SPM_STACK_SIZE);
static struct k_thread spm_thread_data;

#define IRQ_THREAD_PRIORITY	6
#define IRQ_THREAD_STACK_SIZE	4096
static K_THREAD_STACK_DEFINE(irq_stack_area, IRQ_THREAD_STACK_SIZE);
static struct k_thread irq_thread_data;

#define SPM_WAIT_MSEC	1
#define IRQ_WAIT_USEC	10000

#define	BUFFER_LENGTH 4

static void spm_thread(void)
{
	struct device *ent_dev;
	u8_t buffer[BUFFER_LENGTH] = { 0 };

	ent_dev = device_get_binding(CONFIG_ENTROPY_NAME);
	if (!ent_dev) {
		printk("error: no entropy device\n");
		return;
	}

	printk("Entropy device: %s\n", ent_dev->config->name);

	while (1) {
		entropy_get_entropy(ent_dev, buffer, BUFFER_LENGTH);
		k_sleep(K_MSEC(SPM_WAIT_MSEC));
	}
}

static K_SEM_DEFINE(sem_irq, 0, 1);

static struct device *counter_dev;
static struct counter_alarm_cfg alarm_cfg;

static void counter_interrupt_fn(struct device *counter_dev, u8_t chan_id,
				 u32_t ticks, void *user_data)
{
	/* Restart counter. */
	counter_set_channel_alarm(counter_dev, 0, user_data);

	k_sem_give(&sem_irq);
}

static void irq_thread(void)
{
	int err;
	u32_t ticks;
	u32_t seq = 0;

	/* Initialize counter. */
	counter_dev = device_get_binding("TIMER_0");
	if (!counter_dev) {
		printk("Failed to get timer\n");
		return;
	}

	ticks = counter_us_to_ticks(counter_dev, IRQ_WAIT_USEC);

	/* Configure alarm. */
	alarm_cfg.ticks = ticks;
	alarm_cfg.callback = counter_interrupt_fn;
	alarm_cfg.user_data = &alarm_cfg;

	/* Start counter. */
	err = counter_start(counter_dev);
	if (err != 0) {
		printk("counter_start: %d\n", err);
		return;
	}

	/* Start Channel alarm. */
	err = counter_set_channel_alarm(counter_dev, 0, &alarm_cfg);
	if (err != 0) {
		printk("counter_set_channel_alarm: %d\n", err);
		return;
	}

	while (1) {
		/* Wait for IRQ. */
		k_sem_take(&sem_irq, K_FOREVER);
		seq++;
		if (!(seq%100)) {
			printk("Seq: %u\n", seq);
		}
	}
}

void main(void)
{
	printk("Start test !\n");

	/* Start SPM thread. */
	k_thread_create(&spm_thread_data, spm_stack_area,
			K_THREAD_STACK_SIZEOF(spm_stack_area),
			(k_thread_entry_t)spm_thread,
			NULL, NULL, NULL,
			SPM_PRIORITY, 0, K_NO_WAIT);

	/* Start IRQ thread. */
	k_thread_create(&irq_thread_data, irq_stack_area,
			K_THREAD_STACK_SIZEOF(irq_stack_area),
			(k_thread_entry_t)irq_thread,
			NULL, NULL, NULL,
			IRQ_THREAD_PRIORITY, 0, K_NO_WAIT);
}
